#!/bin/bash

############################################################################################
# COPYRIGHT LICENSE:  This utility is owned by IBM and is provided in source code form. 
# You may copy and use (but not modify)  the utility  without payment to IBM for the 
# purposes of helping you to migrate your data from an IBM Workload Deployer product to
# another IBM product. Notwithstanding anything to the contrary,  IBM PROVIDES THE UTILITY
# ON AN "AS IS" BASIS AND IBM DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING,
# BUT NOT LIMITED TO, ANY IMPLIED WARRANTIES OR CONDITIONS OF MERCHANTABILITY, SATISFACTORY
# QUALITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND ANY WARRANTY OR CONDITION OF
# NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL
# OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR OPERATION OF THE UTLITY.  
# IBM MAY PROVIDE LIMITED SUPPORTFOR THE UTLITY, BUT IBM HAS NO OBLIGATION TO PROVIDE
# MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS TO THE UTLITITY.
###########################################################################################

DATA_FILE="migration_data.txt"
MIGRATION_MODULE="pureappmigrator.tar.gz"

COMMONS="commons.sh"
source $COMMONS

while [ $# -gt 0 ]
do
	if [ -z "$2" ] ; then
    	echo "No argument supplied for option $1. Exiting the script."
    	exit 2
	fi
	case $1 in
			-f*|--file*)
				DATA_FILE=$2
				shift 2
				;;
			-*)
				echo "WARN: Unknown option (ignored): $1" 1>&2
				shift
				;;
			*)
				echo "Unknow option: $1. Exiting the script."
				exit 2
				;;
	esac
done

echo "--------------------------------------------"
echo "Install tool started"
echo "--------------------------------------------"

if [ ! -e "$DATA_FILE" ] ; then
        echo "Property file $DATA_FILE doesn't exist. Please provide valid path by specyfing -f option"
        exit
fi

source $DATA_FILE
INSTALL_DIR=$(dirname "$0")


echo "Installing migration tool ..." ; echo ""		
verifyIWD3107Credentials
re=$?
if [ $re -eq 0 ] ; then
	echo "Connecting to IWD server ..."
	$IWD_CLI/bin/deployer -h $IWD_3107_HOSTNAME -u $IWD_3107_USER -p $IWD_3107_PASSWORD -a -c "print \"OK\""
	re=`echo $?`
	if [ "$re" -ne "0" ] ; then
		echo "Failed to connect to IWD using the credentials provided, cannot continue."
		exit 1
	fi	
fi
echo""

verifyPureAppCredentials
re=$?
if [ $re -eq 0 ] ; then
	echo "Connecting to PureApp server ..."
	$IWD_CLI/bin/deployer -h $PUREAPP_HOSTNAME -u $PUREAPP_USER -p $PUREAPP_PASSWORD -i -a -c "print \"OK\""
	re=`echo $?`
	if [ "$re" -ne "0" ] ; then
		echo "Failed to connect to PureApp using the credentials provided, cannot continue."
		exit 1
	fi
fi
echo ""


echo "Installing migration module ..."
for dir in $(ls -d ${IWD_CLI}/lib/*/); do
	tar -zxf $INSTALL_DIR/$MIGRATION_MODULE -C $dir
	re=`echo $?`
	if [ "$re" -eq "0" ] ; then
		echo "Installed to $dir successfully"
	else
		echo "Error encountered while extracting migration module to $dir"
		exit 1
	fi
done
echo ""
echo "Installation completed"	
echo ""

exit 0