'''
COPYRIGHT LICENSE:  This utility is owned by IBM and is provided in source code form. 
You may copy and use (but not modify)  the utility  without payment to IBM for the 
purposes of helping you to migrate your data from an IBM Workload Deployer product to
another IBM product. Notwithstanding anything to the contrary,  IBM PROVIDES THE UTILITY
ON AN "AS IS" BASIS AND IBM DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING,
BUT NOT LIMITED TO, ANY IMPLIED WARRANTIES OR CONDITIONS OF MERCHANTABILITY, SATISFACTORY
QUALITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND ANY WARRANTY OR CONDITION OF
NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR OPERATION OF THE UTLITY.  
IBM MAY PROVIDE LIMITED SUPPORTFOR THE UTLITY, BUT IBM HAS NO OBLIGATION TO PROVIDE
MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS TO THE UTLITITY.
'''

import codecs
import getopt
import sys
import os
import subprocess
import re
import base64
import httplib
import time
import shutil
from xml.dom import minidom
from deployer import ziputils
from deployer.messages import message
from deployer import facetutils, http, patterntopython, prettify, utils, validators, patternloader, patternutils
from pureappmigrator import *
from pureappmigrator import constants, script_packages, virtual_system_patterns, migration_exceptions

from java.io import File

def prepareArtifactsForMigration(artifacts, artifactList):
	if artifacts == "all" :
		for type in artifactTypes:
			artifactList[type] = 1
	else:	
		for type in artifactTypes:
			for artifact in artifacts:
				if type == artifact:
					artifactList[type] = 1
	print "Artifacts to migrate: %s" % (str(artifacts))	

def migrateArtifacts(dest,artifactList,mode, options):	
	timestamp = time.strftime("%m-%d_%H%M%S", time.localtime())
	logFile = mode + "_" + timestamp + ".log"
	tmpStdout = sys.stdout
	tmpStderr =	sys.stderr
	f = initLog(logFile)
	
	try:
		if mode == "export" and checkDiskSpace(dest,artifactList,options):
			exportArtifacts(dest, artifactList, options)
		elif mode == "import":
			importArtifacts(dest, artifactList, options)
	finally:		
		f.close()
		sys.stdout = tmpStdout
		sys.stderr = tmpStderr

def exportArtifacts(dest, artifactList, options):
	try:
		print "Exporting artifacts to %s" % (dest)
		print "--------------------------------------------\n"
		if artifactList['script']:		
			script_packages.exportScriptPackages(dest)		
		if artifactList['addon']:
			add_ons.exportAddOns(dest)		
		if artifactList['virtimg']:
			virtual_images.exportVirtualImages(dest)
		if artifactList['vsys']:
			virtual_system_patterns.exportVirtualSystemPatterns(dest)
		if artifactList['ptype']:
			pattern_types.exportPatternTypes(dest)
		if artifactList['efix']:
			emergency_fixes.exportEmergencyFixes(dest, options)		
		if artifactList['vapp']:
			virtual_application_patterns.exportVirtualApplicationPatterns(dest)	
		print "--------------------------------------------"
		print "Export of artifacts completed\n"
	except Exception, err:
		migration_exceptions.printGeneralException(err, str(err))
		print "Export of artifacts failed\n"
	
def importArtifacts(dest, artifactList, options):
	try:
		print "Importing artifacts from %s\n" % (dest)
		print "--------------------------------------------\n"
		if artifactList['script'] == 1 :
			importArtifact ('script',options)
		if artifactList['addon'] == 1 :
			importArtifact ('addon',options)
		if artifactList['virtimg'] == 1 :
			importArtifact ('virtimg',options)
		if artifactList['efix'] == 1 :
			importArtifact ('efix',options)
		if artifactList['vsys'] == 1 :
			importArtifact ('vsys',options)
		if artifactList['ptype'] == 1 :
			importArtifact ('ptype',options)
		if artifactList['vapp'] == 1 :
			importArtifact ('vapp',options)	
		print "--------------------------------------------"
		print "Import of artifacts completed\n"
	except Exception, err:
		migration_exceptions.printGeneralException(err, str(err))
		print "Import of artifacts failed\n"

def importArtifact(artifact, options=None):
	if artifact == 'script':		
		re = script_packages.importScriptPackages(dest,options)		
	elif artifact == 'addon' :
		re = add_ons.importAddOns(dest,options)		
	elif artifact == 'virtimg':
		re = virtual_images.importVirtualImages(dest,options)
	elif artifact == 'vsys':
		re = virtual_system_patterns.importVirtualSystemPatterns(dest,options)
	elif artifact == 'efix':
		re = emergency_fixes.importEmergencyFixes(dest,options)
	elif artifact == 'ptype':
		re = pattern_types.importPatternTypes(dest)	
	elif artifact == 'vapp':
		re = virtual_application_patterns.importVirtualApplicationPatterns(dest,options)
	
	if re != 0:
		print "Error while importing artifacts. Import stopped !"
		sys.exit(2)

def checkDiskSpace(dest, artifactList, options):
	freeSpace = calculateFreSpace(dest)
	print 'Free space : %d MB' % (freeSpace)
	if artifactList['virtimg'] == 1 :
		requiredSpace = calculateRequiredSpace(options['iwd_3107_hostname'],constants.IWD_3107_HTTPS_PORT,options['iwd_3107_user'],options['iwd_3107_password'] )
		print 'Minimum required space for virtual images: %d MB' % (requiredSpace)
		if requiredSpace >= freeSpace:
			print 'Required space is larger than available on the disk. Exiting\n'
			return False
	return True
		
def calculateFreSpace(dest):
	path = dest
	if not os.path.exists(path):
		patternloader._createDir(path)
	
	try:
		diskPartition = File(dest)	
		freePartitionSpace = diskPartition.getFreeSpace();
		return freePartitionSpace / 1024 / 1024
	except Exception, err:
		return -1
		
def calculateRequiredSpace(host, port, username, password):
	print "Calculating required space ..."
	url = "/resources/virtualImages/IMAGE_ID/viovf"
	totalSize = long(0)
	for image in deployer.virtualimages:
		id = str(image.id)		
		uri = re.sub("IMAGE_ID",id,url)
		resp = rest.iwdRestRequest(host,port, username, password, "GET", uri, None)
		if resp['status'] == rest.STATUS_GET_SUCCESS :
			ovf = patternloader._parseJSON(resp['response'])			
			if ovf != None and ovf[0]['ovfXML'] != None:
				imgSize = long(0)
				xmldoc = minidom.parseString(ovf[0]['ovfXML'])
				fileList = xmldoc.getElementsByTagName('File')
				if len(fileList) == 0:
					fileList = xmldoc.getElementsByTagName('ovf:File')
					if len(fileList) == 0:
						print "\tUnable to get size of the image %s" % (image.name)
						continue				
				for file in fileList:						
					if file.attributes.has_key('size'):
						size = long(file.attributes['size'].value)
					elif file.attributes.has_key('ovf:size'):
						size = long(file.attributes['ovf:size'].value)
					if size :
						imgSize = imgSize + size
				if imgSize > 0 :
					print "\tImage size for %s is %d MB" %(image.name, (imgSize /1024/1024))
					totalSize = totalSize + imgSize
				else:
					print "\tUnable to get size of the image %s" % (image.name)
							
		else:
			print '\tUnable to get size of the image %s' % (image.name)
	return totalSize / 1024 / 1024

def initLog(logFile):
	currentDir = os.path.dirname(os.path.realpath(sys.argv[0]))
	logDir = currentDir + "/logs"
	if not os.path.exists(logDir):
		os.makedirs(logDir)
	logFileFqp = logDir + "/" + logFile  
	print "Writing output to : %s\n" % (logFileFqp)
	f = open(logFileFqp,'w')
	sys.stdout = f
	sys.stderr = f
	return f
	
	
dest = None
mode = None
artifacts = None
options = None
migrationModes = ['import','export']
artifactTypes = ['script', 'addon', 'virtimg', 'vsys', 'ptype', 'efix', 'vapp']
artifactList = {'script':0, 'addon':0, 'virtimg':0, 'vsys':0, 'ptype':0, 'efix':0, 'vapp':0}	

try:
    (params, args) = getopt.getopt(sys.argv[1:], 'm:d:a', ['mode=', 'data=', 'artifacts='])
except getopt.GetoptError:
    help()

for param in params:
	if param[0] == '-d' or param[0] == '--data':
		options = patternloader._parseJSON(param[1])
		dest = options['dest']
	elif param[0] == '-m' or param[0] == '--mode':        
		mode = param[1]
		if not mode in migrationModes :
			print "Invalid mode. Please choose export or import mode (-m export|import)."
			sys.exit(1)
	elif param[0] == '--artifacts':
		artifacts = param[1]
		if artifacts == None or artifacts == "" :
			print "No artifacts specified, importing|exporting all."
			artifacts = 'all'
		if artifacts != 'all' :
			artifacts = artifacts.split(',')
			if len(artifacts):
				for a in artifacts:
					if not a in artifactTypes:
						print "Invalid artifact type: %s. Please provide valid parameter from list %s" %(a, str(artifactTypes))
						sys.exit(1)
			else:
				print 'Empty aritfacts list. Please provide valid parameters from list %s' %(str(artifactTypes))
				sys.exit(1)
		
if dest == None or dest == "" :
	print "Migration directory was not specified. Exiting"
	sys.exit(1)
if artifacts == None:
	print "Artifacts for migration was not specified. Exiting"
	sys.exit(1)


prepareArtifactsForMigration(artifacts, artifactList)
migrateArtifacts(dest,artifactList,mode, options)


		
	