#!/bin/bash

############################################################################################
# COPYRIGHT LICENSE:  This utility is owned by IBM and is provided in source code form. 
# You may copy and use (but not modify)  the utility  without payment to IBM for the 
# purposes of helping you to migrate your data from an IBM Workload Deployer product to
# another IBM product. Notwithstanding anything to the contrary,  IBM PROVIDES THE UTILITY
# ON AN "AS IS" BASIS AND IBM DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING,
# BUT NOT LIMITED TO, ANY IMPLIED WARRANTIES OR CONDITIONS OF MERCHANTABILITY, SATISFACTORY
# QUALITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND ANY WARRANTY OR CONDITION OF
# NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL
# OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR OPERATION OF THE UTLITY.  
# IBM MAY PROVIDE LIMITED SUPPORTFOR THE UTLITY, BUT IBM HAS NO OBLIGATION TO PROVIDE
# MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS TO THE UTLITITY.
###########################################################################################

INSTALL_DIR=$(dirname "$0")
DATA_FILE="$INSTALL_DIR/migration_data.txt"
MODE=""
ARTIFACTS=""
MIGRATION_FILE="$INSTALL_DIR/migration.py"

COMMONS="$INSTALL_DIR/commons.sh"
source $COMMONS

usage() { echo "Usage: $0 -m <export|import> [-a <all|script|addon|virtimg|vsys|efix|ptype|vapp>]" 1>&2; exit 1; }

while [ $# -gt 0 ]
do
	if [ -z "$2" ] ; then
    	echo "No argument supplied for option $1. Exiting the script."
    	exit 2
	fi
	case $1 in
			-f*|--file*)
				DATA_FILE=$2
				shift 2
				;;
			-m*|--mode*)
				MODE=$2
				if [ "$MODE" != "export" ] && [ "$MODE" != "import" ]; then
					usage
				fi                
				shift 2
				;;
			-a*|--artifacts*)
				ARTIFACTS=$2
				if [ -z "$ARTIFACTS" ]; then
					ARTIFACTS="all"
				fi
				if [ "$ARTIFACTS" != "all" ] && [ "$ARTIFACTS" != "script" ] && [ "$ARTIFACTS" != "addon" ] && [ "$ARTIFACTS" != "efix" ] && [ "$ARTIFACTS" != "virtimg" ] && [ "$ARTIFACTS" != "vsys" ] && [ "$ARTIFACTS" != "ptype" ] && [ "$ARTIFACTS" != "vapp" ]; then
					usage                
				fi
				shift 2
				;;
			-*)
				echo "WARN: Unknown option (ignored): $1" 1>&2
				shift
				;;
			*)
				usage
				break
				;;
	esac
done

if [ -z "${MODE}" ]; then
	usage
fi

echo "--------------------------------------------"
echo "Migration tool started"
echo "--------------------------------------------"

if [ ! -e "$DATA_FILE" ] ; then
	echo "Property file $DATA_FILE doesn't exist. Please provide valid path by specyfing -f option"
	echo ""
	exit
fi

source $DATA_FILE

OPTIONS="\"dest\":\"$MIGRATION_DEST\""
if [ "$MODE" == "export" ] ; then
	verifyIWD3107Credentials
	OPTIONS="$OPTIONS , \"iwd_3107_hostname\":\"$IWD_3107_HOSTNAME\",
	\"iwd_3107_user\":\"$IWD_3107_USER\",\"iwd_3107_password\":\"$IWD_3107_PASSWORD\""
	OPTIONS="{$OPTIONS}"
	$IWD_CLI/bin/deployer -h $IWD_3107_HOSTNAME -u $IWD_3107_USER -p $IWD_3107_PASSWORD -f $MIGRATION_FILE -m $MODE --artifacts "$ARTIFACTS" -d "$OPTIONS"
elif [ "$MODE" == "import" ] ; then
	verifyPureAppCredentials
	OPTIONS="$OPTIONS ,\"pureapp_hostname\":\"$PUREAPP_HOSTNAME\",
	\"pureapp_user\":\"$PUREAPP_USER\", \"pureapp_password\":\"$PUREAPP_PASSWORD\""
	OPTIONS="{$OPTIONS}"		
	$IWD_CLI/bin/deployer -i -h $PUREAPP_HOSTNAME -u $PUREAPP_USER -p $PUREAPP_PASSWORD -f $MIGRATION_FILE -m $MODE --artifacts "$ARTIFACTS" -d "$OPTIONS"
else
	echo "Invalid mode specified. Available modes: export, import (-m export|import)."
	echo ""
	exit 1
fi		


exit 0