'''
COPYRIGHT LICENSE:  This utility is owned by IBM and is provided in source code form. 
You may copy and use (but not modify)  the utility  without payment to IBM for the 
purposes of helping you to migrate your data from an IBM Workload Deployer product to
another IBM product. Notwithstanding anything to the contrary,  IBM PROVIDES THE UTILITY
ON AN "AS IS" BASIS AND IBM DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING,
BUT NOT LIMITED TO, ANY IMPLIED WARRANTIES OR CONDITIONS OF MERCHANTABILITY, SATISFACTORY
QUALITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND ANY WARRANTY OR CONDITION OF
NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR OPERATION OF THE UTLITY.  
IBM MAY PROVIDE LIMITED SUPPORTFOR THE UTLITY, BUT IBM HAS NO OBLIGATION TO PROVIDE
MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS TO THE UTLITITY.
'''

import codecs
import os
import datetime
import deployer
from deployer import patternloader
from pureappmigrator import rest, constants, migration_exceptions

JSON_FILE = 'addon.json'
RESOURCE_TYPE="addons"

def _recordAddonAttributes(addon):
	addonObj = {patternloader.NAME: addon.name, patternloader.TYPE: addon.type, patternloader.READ_ONLY: addon.isReadOnly()}
	patternloader._recordScriptAttributes(addonObj, addon)
	addonObj['owner'] = addon.owner.username
	return addonObj

def exportAddOns(dest):	
	addonsDir = os.path.join(dest, constants.ADDON_DIR)
	print "Exporting Add-Ons to directory: %s\n" % (addonsDir)
	if os.path.exists(addonsDir) :
		print "Destination %s exists. Please remove the directory before export. Skipping" % (addonsDir)
		print "-----------------\n"
		return 2
	patternloader._createDir(addonsDir)	
	i = 1
	for addon in deployer.addons :
		try:
			addonDir = os.path.join(addonsDir, str(i))
			print "[%d] [%s UTC] Exporting Add-On %s to %s ..." %(i, datetime.datetime.utcnow(), addon.name, addonDir) 
			patternloader._createDir(addonDir)
			addonObj = _recordAddonAttributes(addon)
					
			if addon.filename :
				addon.archive.get(os.path.join(addonDir, addon.filename))
				addonObj[patternloader.FILE] = addon.filename
			
			f = codecs.open(os.path.join(addonDir,JSON_FILE), 'wb', patternloader.UTF_8)
			try:
				f.write(str(patternloader._toJSON4J([addonObj]).serialize(True)))
			finally:
				f.close()	
			print "[%d] Add-On exported\n" % (i)
		except Exception, err:
			migration_exceptions.printExportException("Add-On",err,i)
		i = i + 1
	print "Export of Add-Ons completed"
	print "-----------------\n"

def importAddOns(dest,options=None):
	addonsDir = os.path.join(dest, constants.ADDON_DIR)
	print "Importing Add-Ons from directory: %s\n" % (addonsDir)
	if not os.path.exists(addonsDir) :
		print "Destination %s doesn't exists. Please export this artifact first. Exiting" % (addonsDir)
		print "-----------------\n"
		return 2
	i = 1
	for dir in os.listdir(addonsDir) :
		try:
			addonDir = os.path.join(addonsDir,dir)
			print "[%d] [%s UTC] Processing directory: %s ..." % (i, datetime.datetime.utcnow(), addonDir)
			f = codecs.open(os.path.join(addonDir,JSON_FILE), 'r', patternloader.UTF_8)
			addonPackage = patternloader._parseJSON(f.read())[0]		
			f.close()
			if addonPackage :
				name = addonPackage[patternloader.NAME]
				type = addonPackage[patternloader.TYPE]
				addons = filter(lambda addon: addon.name == name, deployer.addons[name])
				print "[%d] Importing addon Package : %s ..." % (i,name)
				if len(addons) == 0 :	
					addon = deployer.addons << {patternloader.NAME: name, patternloader.TYPE: type}
					addons.append(addon)
					file = addonPackage.get(patternloader.FILE, None)
					if not file is None:
						path = os.path.join(addonDir, file)
						addon.archive.set(path)
						
					patternloader._loadScriptAttributes(addonPackage, addons[0])
					
					addonList = deployer.addons.list({'name': name})
					if len(addonList) == 0 :
							print "[%d] Error while importing the Add-On. Add-On not imported.\n" %(i)
					else:
						print "[%d] Add-On successfully imported.\n" % (i)
				else :
					print "[%d] Add-On %s already exists. Skipping\n" % (i,name)
		except Exception, err:
			migration_exceptions.printImportException("Add-On",err,i)
		i = i+1
	print "Import of Add-Ons completed"
	print "-----------------\n"
	return 0
		
