'''
COPYRIGHT LICENSE:  This utility is owned by IBM and is provided in source code form. 
You may copy and use (but not modify)  the utility  without payment to IBM for the 
purposes of helping you to migrate your data from an IBM Workload Deployer product to
another IBM product. Notwithstanding anything to the contrary,  IBM PROVIDES THE UTILITY
ON AN "AS IS" BASIS AND IBM DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING,
BUT NOT LIMITED TO, ANY IMPLIED WARRANTIES OR CONDITIONS OF MERCHANTABILITY, SATISFACTORY
QUALITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND ANY WARRANTY OR CONDITION OF
NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR OPERATION OF THE UTLITY.  
IBM MAY PROVIDE LIMITED SUPPORTFOR THE UTLITY, BUT IBM HAS NO OBLIGATION TO PROVIDE
MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS TO THE UTLITITY.
'''

import codecs
import os
import datetime
import httplib
import deployer
from deployer import ziputils
from deployer.messages import message
from deployer import facetutils, http, patterntopython, prettify, utils, validators, patternloader, patternutils
from pureappmigrator import rest, constants, migration_exceptions

# constants
JSON_FILE = 'efix.json'
headers = {'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8','Accept-Language': 'en-US,en;q=0.5','Accept-Encoding': 'gzip, deflate','Connection': 'keep-alive', 'User-Agent': 'IBM-Workload-Deployer-CLI/3.1.0'}

# fix attributes
NAME = "name"
DESCRIPTION = "description"
SEVERITY = "severity"
TARGET = "target"
PREREQS = "prereqs"
TYPE = "type"
FILE = "file"

def exportEmergencyFixes(dest,options):	
	efixesDir = os.path.join(dest, constants.EFIX_DIR)
	print "Exporting Emergency Fixes to directory: %s\n" % (efixesDir)
	if os.path.exists(efixesDir) :
		print "Destination %s exists. Please remove the directory before export. Skipping" % (efixesDir)
		print "-----------------\n"
		return 2
	patternloader._createDir(efixesDir)	
	i = 1
	for efix in deployer.fixes :
		try:
			efixDir = os.path.join(efixesDir, str(i))
			print "[%d] [%s UTC] Exporting Emergency Fix %s to %s..." %(i, datetime.datetime.utcnow(), efix.name, efixDir) 
			patternloader._createDir(efixDir)		
			efixObj = _recordFixAttributes(efix)
			filename = _downloadFixPackage(efixDir, efix, options['iwd_3107_hostname'], constants.IWD_3107_HTTPS_PORT)		
			if filename :			
				efixObj[FILE] = filename		
			f = codecs.open(os.path.join(efixDir,JSON_FILE), 'wb', patternloader.UTF_8)
			try:
				f.write(str(patternloader._toJSON4J([efixObj]).serialize(True)))
			finally:
				f.close()	
			print "[%d] Efix exported!\n" % (i)
		except Exception, err:
			migration_exceptions.printExportException("Emergency Fix",err,i)
		i = i + 1
	print "All Emergency Fixes has been exported"
	print "-----------------\n"

def importEmergencyFixes(dest, options=None):
	efixesDir = os.path.join(dest, constants.EFIX_DIR)
	print "Importing  Emergency Fixes from directory: %s" % (efixesDir)
	if not os.path.exists(efixesDir) :
		print "Destination %s doesn't exists. Please export this artifact first. Exiting" % (efixesDir)
		print "-----------------\n"
		return 2
	i = 1
	for dir in os.listdir(efixesDir) :
		try:
			efixDir = os.path.join(efixesDir,dir)
			print "[%d] [%s UTC] Processing directory: %s ..." % (i, datetime.datetime.utcnow(), efixDir)
			f = codecs.open(os.path.join(efixDir,JSON_FILE), 'r', patternloader.UTF_8)
			efixPackage = patternloader._parseJSON(f.read())[0]		
			f.close()
			if efixPackage:
				name = efixPackage[patternloader.NAME]
				efixes = filter(lambda fix: fix.name == name, deployer.fixes[name])
				print "[%d] Emergency Fix: %s ..." % (i,name)
				if len(efixes) == 0 :
					efix = deployer.fixes << {patternloader.NAME: name}
					efixes.append(efix)
					file = efixPackage.get(patternloader.FILE, None)
					if not file is None:
						path = os.path.join(efixDir, file)
						efix.archive.set(path)
				
					patternloader._loadScriptAttributes(efixPackage, efixes[0])
					
					efixList = deployer.fixes.list({'name': name})
					if len(efixList) == 0 :
							print "[%d] Error while importing the Emergency Fix. Emergency Fix not imported.\n" %(i)
					else:
						print "[%d] Emergency Fix successfully imported.\n" % (i)
				else :
					print "[%d] Emergency Fix %s already exists. Skipping\n" % (i,name)
		except Exception, err:
			migration_exceptions.printImportException("Emergency Fix",err,i)
		i = i+1
	print "Import of Emergency Fixes completed"
	print "-----------------\n"		
	return 0		
	
def _recordFixAttributes(efix):  
	efixObj = {NAME: efix.name}       
	if efix.description != None:
		efixObj[DESCRIPTION] = efix.description
	prereqs = _getFixPrereqs(efix)
	if len(prereqs):
		efixObj[PREREQS] = prereqs
	if efix.severity != None:
		efixObj[SEVERITY] = efix.severity
	if efix.type != None:
		efixObj[TYPE] = efix.type
	if efix.target != None:
		efixObj[TARGET] = efix.target
	return efixObj

def _getFixPrereqs(efix):
	prereqs = []
	i = 0
	while True:
		try:
			prereq = efix.prereqs[i]			
			if('imagename' in prereq.keys()):				
				image = {
				  'imagename': prereq['imagename'],
				  'version': prereq['version'],
				  'operatingsystemid': prereq['operatingsystemid'],
				  'operatingsystemversion': prereq['operatingsystemversion'] 
				}
				prereqs.append(image)				
			elif('pluginname' in prereq.keys()):				
				plugin = { 'name': prereq['pluginname'] + '/' + prereq['pluginvrmf'] }
				prereqs.append(plugin)
			i = i+1
		except:
			break
	return prereqs
	
def _downloadFixPackage(efixDir, efix, host, port):
	filename = None
	if efix.uri :
		conn = httplib.HTTPSConnection(host, port)
		targetHeaders = http._authHeader(headers)
		method = 'GET'
		uri = efix.uri+'/scriptArchive.zip'
		body = None
		conn.request(method,uri,body,targetHeaders)
		resp = conn.getresponse()
		if resp.status == 200 :
			try:
				content_name = resp.getheader('Content-Disposition').split(';')[1].strip()
				if content_name.split('=')[0] == 'filename' :
					filename = content_name[content_name.find("=")+1:]
					f = codecs.open(os.path.join(efixDir,filename), 'wb')
					try:
						f.write(resp.read())
					finally:
						f.close()
			except:
				print '\t\tError while downloading fix package'			
		conn.close()
	return filename
	

