'''
COPYRIGHT LICENSE:  This utility is owned by IBM and is provided in source code form. 
You may copy and use (but not modify)  the utility  without payment to IBM for the 
purposes of helping you to migrate your data from an IBM Workload Deployer product to
another IBM product. Notwithstanding anything to the contrary,  IBM PROVIDES THE UTILITY
ON AN "AS IS" BASIS AND IBM DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING,
BUT NOT LIMITED TO, ANY IMPLIED WARRANTIES OR CONDITIONS OF MERCHANTABILITY, SATISFACTORY
QUALITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND ANY WARRANTY OR CONDITION OF
NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR OPERATION OF THE UTLITY.  
IBM MAY PROVIDE LIMITED SUPPORTFOR THE UTLITY, BUT IBM HAS NO OBLIGATION TO PROVIDE
MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS TO THE UTLITITY.
'''

import sys
import os
import httplib
import base64
from deployer import patternloader, http


ERROR_RESOURCE_EXISTS="exists"
ERROR_RESOURCE_CREATION="error"

STATUS_GET_SUCCESS = 200
STATUS_CREATED = 201
STATUS_PUT_SUCCESS = 204
STATUS_NOT_AUTHORIZED = 401
STATUS_FORBIDDEN = 403
STATUS_NOT_FOUND = 404
STATUS_CONFILICT = 409
SUCCESS_STATUSES = [STATUS_GET_SUCCESS,STATUS_CREATED,STATUS_PUT_SUCCESS]
ERROR_STATUSES = [STATUS_NOT_AUTHORIZED,STATUS_FORBIDDEN,STATUS_NOT_FOUND,STATUS_CONFILICT]

SCP_HTTPS_PORT=443
IWD_HTTPS_PORT=443

def createKeystoneResource(authObj, resourceType, restObj):
	headers = _getKeystoneHeaders(authObj)
	url = "/" + authObj['endpointId'] + "/v3/" + resourceType	
	parameters = patternloader._toJSON4J(restObj)
	response = _postHttpRequest(authObj['gatewayHost'],authObj['gatewayPort'],url,str(parameters),headers)
	if response['status'] == STATUS_CREATED:		
		response['response'] = patternloader._parseJSON(response['response'])			
	return response
		
def getKeystoneResourceList(authObj, resourceType):
	headers = _getKeystoneHeaders(authObj)
	url = "/" + authObj['endpointId'] + "/v3/" + resourceType
	response = _getHttpRequest(authObj['gatewayHost'],authObj['gatewayPort'],url,headers)
	if response['status'] == STATUS_GET_SUCCESS :
		resourceList = patternloader._parseJSON(response['response'])
		return resourceList[resourceType]
	else:
		return None
		
def getKeystoneResource(authObj, resourceType, resourceName):
	keystoneResourceList = getKeystoneResourceList(authObj, resourceType)	
	keystoneResource = None
	if keystoneResourceList is not None:
		for resource in keystoneResourceList:
			if resource['name'] == resourceName:
				keystoneResource = resource
				break		
		return keystoneResource				
	else:
		return None
		
def putKeystoneRequest(authObj, url):
	headers = _getKeystoneHeaders(authObj)
	response = _putHttpRequest(authObj['gatewayHost'],authObj['gatewayPort'],url, "", headers)
	return response['status']
		
def getKeystoneAuthenticationToken(scoUsername, scoPassword, endpointId, gatewayHost,gatewayPort=9973):
	headers = _getKeystoneStandardHeaders()	
	url = "/" + endpointId + "/v3/auth/tokens"
	parameters =  {"auth": { "identity": { "methods": [ "password" ],
	"password": { "user": { "domain": { "name": "Default" }, "name": scoUsername, "password": scoPassword }}}, 
	"scope": { "project": { "domain": {  "name": "Default" }, "name": "admin"}}}}
	parameters = patternloader._toJSON4J(parameters)
	response = _postHttpRequest(gatewayHost,gatewayPort,url,str(parameters),headers)
	
	if response['status'] != STATUS_CREATED :
		print "Unable to get Authorization Token"
		return None
		
	respHeaders = response['headers']
	token = None
	try:
		for header in respHeaders:
			if header[0] == "x-subject-token":
				token = header[1]
				break
		return token
	except Exception, err:
		print "Unable to get Authorization Token"
		print "Error : %s" % (str(err))
		return None	
		
def getKeystoneEndpointId(gatewayHost,gatewayPort=9973):
	headers = _getKeystoneStandardHeaders()	
	url = "/providers"
	response = _getHttpRequest(gatewayHost,gatewayPort, url, headers)
	if response['status'] != STATUS_GET_SUCCESS:
		print "Unable to get Keystone Api information."
		return None
	providers = patternloader._parseJSON(response['response'])
	endpoints = providers['serviceCatalog'][0]['endpoints']	
	endpointId = None
	try:
		for endpoint in endpoints:
			if endpoint['interface'] == 'public':
				endpointId = endpoint['id']
		return endpointId	
	except:
		print "Unable to get Keysone Api information"
		return None		
	
def _getKeystoneHeaders(authObj):
	return _prepareKeystoneAuthHeaders(authObj['authToken'], _getKeystoneStandardHeaders())	
	
def _getKeystoneStandardHeaders():
	return {"Content-Type":"application/json"}
	
def _prepareKeystoneAuthHeaders(authToken, headers):
	headers["X-Auth-Token"] = authToken
	return headers

	
def iwdRestRequest(host,port, username, password, method, url, body):
	headers = _getIwdHeaders(username, password)
	return _httpsRequest(host,port, method, url, body,headers)

def _getIwdHeaders(username, password):
	headers = _prepareIwdAuthHeaders(username, password, _getIwdStandardHeaders())
	return headers
	
def _getIwdStandardHeaders():
	return {"Content-Type":"application/json", 'User-Agent' : 'IBM-Workload-Deployer-CLI/3.1.0'}
	
def _prepareIwdAuthHeaders(username, password, headers):
	headers['Authorization'] = 'Basic ' + base64.b64encode('%s:%s' % (username.encode('utf-8'), password.encode('utf-8')))
	return headers
	
def _getHttpRequest(host, port, url,headers):	
	body = None
	method = "GET"
	return _httpRequest(host,port, method, url, body, headers)	
	
def _getHttpsRequest(host,port, url, headers):
	body = None
	method = "GET"
	return httpsRequest(host,port, method, url, body, headers)
		
def _putHttpRequest(host, port, url, body, headers):		
	method = "PUT"
	return _httpRequest(host,port, method, url, body, headers)		
	
def _putHttpsRequest(host, port, url, body, headers):		
	method = "PUT"
	return httpsRequest(host,port, method, url, body, headers)		
	
def _postHttpRequest(host, port, url, body, headers):	
	method = "POST"
	return _httpRequest(host,port, method, url, body, headers)
	
def _postHttpsRequest(host, port, url, body, headers):	
	method = "POST"
	return httpsRequest(host,port, method, url, body, headers)	
	
def _httpRequest(host, port, method, url, body, headers):
	connection = _getHTTPConnection(host,port)
	return _standardRestRequest(connection, method, url, body , headers)
	
def _httpsRequest(host, port, method, url, body, headers):	
	connection = _getHTTPSConnection(host,port)
	return _standardRestRequest(connection, method, url, body , headers)
	
def _standardRestRequest(connection, method, url, body , headers):	
	connection.request(method,url,body,headers)
	resp = connection.getresponse()
	status = resp.status
	response = resp.read()
	headers = resp.getheaders()
	connection.close()
	return { "headers" : headers, "status" : status, "response" : response}

def _getHTTPConnection(host, port):
	return httplib.HTTPConnection(host,port)

def _getHTTPSConnection(host,port):
	return httplib.HTTPSConnection(host,port)

