'''
COPYRIGHT LICENSE:  This utility is owned by IBM and is provided in source code form. 
You may copy and use (but not modify)  the utility  without payment to IBM for the 
purposes of helping you to migrate your data from an IBM Workload Deployer product to
another IBM product. Notwithstanding anything to the contrary,  IBM PROVIDES THE UTILITY
ON AN "AS IS" BASIS AND IBM DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING,
BUT NOT LIMITED TO, ANY IMPLIED WARRANTIES OR CONDITIONS OF MERCHANTABILITY, SATISFACTORY
QUALITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND ANY WARRANTY OR CONDITION OF
NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR OPERATION OF THE UTLITY.  
IBM MAY PROVIDE LIMITED SUPPORTFOR THE UTLITY, BUT IBM HAS NO OBLIGATION TO PROVIDE
MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS TO THE UTLITITY.
'''

import os
import codecs
import datetime
import deployer
from deployer.messages import message
from deployer import patternloader

from pureappmigrator import rest, constants, migration_exceptions

JSON_FILE = "vapp.json"
MODEL_FILE = "model.json"
RESOURCE_TYPE="applicationPatterns"

def _recordVappAttributes(vapp):
	vappObj={"app_name": vapp.app_name}
	vappObj['app_type'] =  vapp.app_type
	vappObj['patterntype'] = vapp.patterntype
	vappObj['version'] = vapp.version
	vappObj['creator'] = vapp.creator
	return vappObj

def exportVirtualApplicationPatterns(dest):	
	vappsDir = os.path.join(dest, constants.VAPP_DIR)
	print "Exporting Virtual Application Patterns to: %s\n" % (vappsDir)
	if os.path.exists(vappsDir) :
		print "Destination %s exists. Please remove the directory before export. Skipping" % (vappsDir)
		print "-----------------\n"
		return 2
	patternloader._createDir(vappsDir)
	i=1
	for vapp in deployer.applications :		
		vappDir = os.path.join(vappsDir, str(i))
		print "[%d] [%s UTC] Exporting: %s to %s ..." % (i, datetime.datetime.utcnow(), vapp.app_name, vappDir)
		try:
			patternloader._createDir(vappDir)		
			vapp.download(os.path.join(vappDir,MODEL_FILE))
			dirList = os.listdir(vappDir)
			if len(dirList):
				vappObj = _recordVappAttributes(vapp)
				f = codecs.open(os.path.join(vappDir,JSON_FILE), 'wb', patternloader.UTF_8)
				try:
					f.write(str(patternloader._toJSON4J([vappObj]).serialize(True)))
				finally:
					f.close()				
				print "[%d] Virtual Application Pattern exported\n" % (i)
			else:
				print "[%d] Error while exporting Virtuall Application Pattern. Not exported" % (i)
		except Exception, err:
			migration_exceptions.printExportException("Virtual Application Pattern",err,i)
		i = i+ 1
	print "Virtual Application Patterns export completed"
	print "-----------------\n"

def importVirtualApplicationPatterns(dest, options):	
	vappsDir = os.path.join(dest, constants.VAPP_DIR)
	print "Importing Virtual Application Patterns from directory: %s\n" %(vappsDir)
	if not os.path.exists(vappsDir) :
		print "Destination %s doesn't exists. Please export this artifact first. Exiting" % (vappsDir)
		print "-----------------\n"
		return 2
	i = 1
	for dir in os.listdir(vappsDir) :
		try:
			vappDir = os.path.join(vappsDir,dir)
			print "[%d] [%s UTC] Processing directory: %s ..." % (i, datetime.datetime.utcnow(), vappDir)
			vappPath = os.path.join(vappDir,MODEL_FILE)
			f = codecs.open(os.path.join(vappDir,JSON_FILE), 'r', patternloader.UTF_8)
			vappPattern = patternloader._parseJSON(f.read())[0]
			f.close()		
			print "[%d] Importing: %s(app_type:%s) ..." % (i,vappPattern['app_name'],vappPattern['app_type'])	
			retrycounter = 0
			vappList = []
			while retrycounter < 10 :
				try:
					vappList = deployer.applications.list({'app_name':vappPattern['app_name'], 'app_type':vappPattern['app_type']})
					break
				except Exception, err:
					print "\tError retrieving list of virtual application patterns, exception: %s" %(str(err))
					if retrycounter > 9:
						raise
					retrycounter = retrycounter + 1
					print "\tRetrying attempt %d of 10 in 30 seconds" %(retrycounter)
					time.sleep(30)
			if len(vappList) > 0 :
				print "[%d] Virtual Application Pattern already exists. Skipping\n" % (i)
			else:
				deployer.applications.create(vappPath)	
				vappList = deployer.applications.list({'app_name':vappPattern['app_name'], 'app_type':vappPattern['app_type']})
				if len(vappList) == 0 :
						print "[%d] Error while importing the Virtual Application Pattern. Pattern not imported.\n" %(i)
				else:
						print "[%d] Virtual Application Pattern successfully imported.\n" %(i)
		except Exception, err:
			migration_exceptions.printImportException("Virtual Application Pattern",err,i)
		i = i+1
	print "Import of Virtual Application Patterns completed"
	print "-----------------\n"
	return 0


