'''
COPYRIGHT LICENSE:  This utility is owned by IBM and is provided in source code form. 
You may copy and use (but not modify)  the utility  without payment to IBM for the 
purposes of helping you to migrate your data from an IBM Workload Deployer product to
another IBM product. Notwithstanding anything to the contrary,  IBM PROVIDES THE UTILITY
ON AN "AS IS" BASIS AND IBM DISCLAIMS ALL WARRANTIES, EXPRESS OR IMPLIED, INCLUDING,
BUT NOT LIMITED TO, ANY IMPLIED WARRANTIES OR CONDITIONS OF MERCHANTABILITY, SATISFACTORY
QUALITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND ANY WARRANTY OR CONDITION OF
NON-INFRINGEMENT. IBM SHALL NOT BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL
OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR OPERATION OF THE UTLITY.  
IBM MAY PROVIDE LIMITED SUPPORTFOR THE UTLITY, BUT IBM HAS NO OBLIGATION TO PROVIDE
MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS TO THE UTLITITY.
'''

import os
import deployer
from deployer import utils, ziputils, patternloader
import time
import datetime
import base64
import httplib
import codecs
from pureappmigrator import rest, constants, migration_exceptions

JSON_FILE = "virtimg.json"
RESOURCE_TYPE="templates"
port = 443

def _recordImageAttributes(image):
	imageObj={patternloader.NAME: image.name, "version": image.version, "build": image.build, "licenseaccepted" : image.licenseaccepted }
	imageObj['owner'] = image.owner.username
	return imageObj

def exportVirtualImages(dest):	
	virtImgsDir = os.path.join(dest, constants.VIRT_DIR)
	print "Exporting Virtual Images to: %s" % (virtImgsDir)
	print "This operation may take a while, so be patient\n"
	if os.path.exists(virtImgsDir) :
		print "Destination %s exists. Please remove the directory before export. Skipping" % (virtImgsDir)
		print "-----------------\n"
		return 2
	patternloader._createDir(virtImgsDir)
	i = 0
	for image in deployer.virtualimages :
		try:
			i = i + 1
			virtImgDir = os.path.join(virtImgsDir, str(i))
			print '[%d] [%s UTC] Exporting Virtual Image %s to %s ...' % (i, datetime.datetime.utcnow(), image.name, virtImgDir)
			imgStatus = image.currentstatus
			excludeStatusList = ['capturing', 'deleting', 'exporting', 'failed', 'processing', 'queued']
			if constants.VIRT_IMAGE_STATUS[imgStatus] in excludeStatusList :
				print '[%d] Skipping export of virtual image with status: %s\n' % (i, constants.VIRT_IMAGE_STATUS[imgStatus])
				continue
			patternloader._createDir(virtImgDir)
			imageObj = _recordImageAttributes(image)
			image.export(virtImgDir)				
			time.sleep(5)
			# Loop was developed for asynchronuous export.
			# If we export virt image for the same machine export is run synchronously.
			# Below loop will be run just one.
			counter = 0		
			while counter < 1000 :
				curImg = filter(lambda virtimg: virtimg.id == image.id, deployer.virtualimages)[0]
				status = curImg.currentstatus
				if constants.VIRT_IMAGE_STATUS[status] == 'exporting' :
					counter = counter + 1
					print '\tStatus %s. Image not yet exported. Sleeping for 30 sec. Counter = %d' % (curImg.currentstatus_text,counter)
					time.sleep(30)
				elif status == imgStatus :
					imgName = os.listdir(virtImgDir)[0]
					if imgName:
						imageObj['filename'] = imgName
						f = codecs.open(os.path.join(virtImgDir,JSON_FILE), 'wb', patternloader.UTF_8)
						try:
							f.write(str(patternloader._toJSON4J([imageObj]).serialize(True)))
						finally:
							f.close()	
						print '[%d] Image has been exported\n' % (i)
					else:
						print '[%d] Error while exporting Virtual Image. Exported OVA does not exists.' %(i)
					break
				else:
					print "[%d] Error while exporting Virtual Image. Export stopped. Image status : %s\n" % (i,curImg.currentstatus_text)
					print "[%d] Message: %s\n" % (i, curImg.currentmessage)					
		except Exception, err:			
			migration_exceptions.printExportException("Virtual Image",err,i)
	print "Export of Virtual Images completed"
	print "-----------------\n"

def importVirtualImages(dest,options=None):
	virtImgsDir = os.path.join(dest, constants.VIRT_DIR)
	print "Importing Virtual Images from directory %s\n" % (virtImgsDir)
	if not os.path.exists(virtImgsDir) :
		print "Destination %s does not exist. Please export this artifact first. Exiting..." % (virtImgsDir)
		print "-----------------\n"
		return 2
	i = 0
	for dir in os.listdir(virtImgsDir) :
		try:
			i = i+1
			virtImgDir = os.path.join(virtImgsDir,dir)
			print "[%d] [%s UTC] Processing directory: %s ..." % (i, datetime.datetime.utcnow(), virtImgDir)
			if not os.path.exists(os.path.join(virtImgDir, JSON_FILE)) :
				print "[%d] Virtual image metadata file %s does not exist. Skipping...\n" % (i, os.path.join(virtImgDir, JSON_FILE))
				continue
			f = codecs.open(os.path.join(virtImgDir,JSON_FILE), 'r', patternloader.UTF_8)
			virtImage = patternloader._parseJSON(f.read())[0]		
			f.close()
	
			# check if there is an existing image with the same name, version, build		
			matchingImgs = filter(lambda virtimg: virtimg.name == virtImage['name'] and virtimg.version == virtImage['version'] and virtimg.build == virtImage['build'], deployer.virtualimages)
			if len(matchingImgs) > 0:
				print "[%d] Virtual image with the same name, version, and build (%s, %s, %s) already exists. Skipping ...\n" % (i, virtImage['name'], virtImage['version'], virtImage['build'])
				continue
				
			imgName = virtImage['filename']
			if imgName:
				print "[%d] Importing: %s ..." % (i, imgName)
				image = deployer.virtualimages.create(os.path.join(virtImgDir,imgName))			
				time.sleep(5)
				counter = 0
				retrycounter = 0
				while counter < 1000 :
					try:
						counter = counter + 1
						virtImgs = filter(lambda virtimg: virtimg.id == image.id, deployer.virtualimages)
						status = virtImgs[0].currentstatus
						if constants.VIRT_IMAGE_STATUS[status] in ['processing', 'queued']:
							print "\tImport status: %s. Sleeping for 30 seconds" % (virtImgs[0].currentstatus_text)
							retrycounter = 0
							time.sleep(30)
						elif constants.VIRT_IMAGE_STATUS[status] != 'failed' :
							print "[%d] Virtual Image successfully imported, status:%s" %(i, virtImgs[0].currentstatus_text)
							if constants.VIRT_IMAGE_STATUS[status] == 'accept_license' and virtImage['licenseaccepted'] == "T":
								print "[%d] Accepting License ..." %(i)
								virtImgs[0].acceptLicense()
								if constants.VIRT_IMAGE_STATUS[virtImgs[0].currentstatus] != 'accept_license' :
									print "[%d] License accepted" %(i)
								else:
									print "[%d] Error while accepting the license" %(i)
							print "\n"
							break
						else: 
							print "[%d] Virtual Image import failed: %s" %(i, virtImgs[0].currentmessage)
							if virtImgs[0].currentmessage.find('IBM:POWER:AIXLINUX') != -1:
								print "[%d] Deleting unsupported virtual image" %(i)
								virtImgs[0].delete()
								print "[%d] Virtual image deleted" %(i)
							elif virtImgs[0].currentmessage.find('DNZEMW374E') != -1:
								print "[%d] Deleting unsupported virtual image" %(i)
								virtImgs[0].delete()
								print "[%d] Virtual image deleted" %(i)
							print "\n"
							break
					except Exception, err:
						print "\tError retrieving status of import image, exception: %s" %(str(err))
						if retrycounter > 9:
							raise
						retrycounter = retrycounter + 1
						print "\tRetrying attempt %d of 10 in 30 seconds" %(retrycounter)
						time.sleep(30)
		except Exception, err:
			migration_exceptions.printImportException("Virtual Image",err,i)
		except java.io.IOException, err:
			migration_exceptions.printImportException("Virtual Image",err,i)
	print "Import of Virtual Images completed"
	print "-----------------\n"
	return 0
	
